//////////////////////////////////////////////////////////
// Coordinate Transformation (Rotation and Translation) //
//////////////////////////////////////////////////////////

//////////////////////////
// Include header files //
//////////////////////////
#include	<math.h>
#include	<string.h>
#include	"ErrorCodes.h"
#include	"ArgumentConstants.h"
#include	"Coord_Trans.h"

/////////////////////////////
// Perform single rotation //
/////////////////////////////
double				CalcSingleRot(
double				ang,
double				var1,
double				var2,
int					mult)
{
	//Convert degrees to radians
	double	radAng = ang / 180 * kPI;

	//Calculate single rotation
	double	newAng = var1 * cos(radAng) + mult * var2 * sin(radAng);

	//Return result
	return(newAng);
}

///////////////////////////////////
// Rotate around particular axis //
///////////////////////////////////
void				RotateAroundAxis(
double* const		pDinv,
double* const		pDreg,
double				dAng)
{
	double	temp = *pDinv;
	*pDinv = CalcSingleRot(dAng, *pDinv, *pDreg, -1);
	*pDreg = CalcSingleRot(dAng, *pDreg, temp, 1);
}

//////////////////////////////////////
// Rotate coordinates around origin //
//////////////////////////////////////
void				CoordRot(
const FloatVecPtr	vFrom,
const FloatVecPtr	vTo,
const FloatVecPtr	vDist,
const char* const	rotord)
{
	//xyz rotation order
	if (strcmp(rotord, rotords[0]) == 0)
	{
		*vTo = *vFrom;
		RotateAroundAxis(&vTo->y, &vTo->z, vDist->x);
		RotateAroundAxis(&vTo->z, &vTo->x, vDist->y);
		RotateAroundAxis(&vTo->x, &vTo->y, vDist->z);
	}

	//xzy rotation order
	else if (strcmp(rotord, rotords[1]) == 0)
	{
		*vTo = *vFrom;
		RotateAroundAxis(&vTo->y, &vTo->z, vDist->x);
		RotateAroundAxis(&vTo->x, &vTo->y, vDist->z);
		RotateAroundAxis(&vTo->z, &vTo->x, vDist->y);
	}

	//yxz rotation order
	else if (strcmp(rotord, rotords[2]) == 0)
	{
		*vTo = *vFrom;
		RotateAroundAxis(&vTo->z, &vTo->x, vDist->y);
		RotateAroundAxis(&vTo->y, &vTo->z, vDist->x);
		RotateAroundAxis(&vTo->x, &vTo->y, vDist->z);
	}

	//yzx rotation order
	else if (strcmp(rotord, rotords[3]) == 0)
	{
		*vTo = *vFrom;
		RotateAroundAxis(&vTo->z, &vTo->x, vDist->y);
		RotateAroundAxis(&vTo->x, &vTo->y, vDist->z);
		RotateAroundAxis(&vTo->y, &vTo->z, vDist->x);
	}

	//zxy rotation order
	else if (strcmp(rotord, rotords[4]) == 0)
	{
		*vTo = *vFrom;
		RotateAroundAxis(&vTo->x, &vTo->y, vDist->z);
		RotateAroundAxis(&vTo->y, &vTo->z, vDist->x);
		RotateAroundAxis(&vTo->z, &vTo->x, vDist->y);
	}

	//zyx rotation order
	else if (strcmp(rotord, rotords[5]) == 0)
	{
		*vTo = *vFrom;
		RotateAroundAxis(&vTo->x, &vTo->y, vDist->z);
		RotateAroundAxis(&vTo->z, &vTo->x, vDist->y);
		RotateAroundAxis(&vTo->y, &vTo->z, vDist->x);
	}

	//Unknown rotation order; Do nothing
	else
	{
		*vTo = *vFrom;
	}
}

///////////////////////////
// Translate coordinates //
///////////////////////////
void				CoordTrans(
const FloatVecPtr	vFrom,
const FloatVecPtr	vTo,
const FloatVecPtr	vDist)
{
	*vTo = *vFrom;
}
