///////////////////////////////////////////////////////
//                                                   //
// Coordinate Transformer                            //
// for rotating BeamNG.drive jbeams about the origin //
//                                                   //
///////////////////////////////////////////////////////

///////////////////
// Include Files //
///////////////////
#include	<stdio.h>
#include	<stdlib.h>
#include	<string.h>
#include	"ErrorCodes.h"
#include	"ArgumentConstants.h"
#include	"JBEAM_Transformer.h"
#include	"DAE_Transformer.h"

/////////////////////////////
// Validate Rotation Angle //
/////////////////////////////
int		validateRotation(
char*	sRot)
{
	double	dRot = atof(sRot);
	if (dRot < 360.0 && dRot > -360.0) return(gecNoError);
	printf("Invalid rotation angle\r\n");
	return(gecInvAng);
}

////////////////////////
// Validate Arguments //
////////////////////////
int		validateArgs(
int		argc,
char*	argv[])
{
	//Check argument count
	if (argc != argCount)
	{
		printf("Wrong number of arguments.\r\n");
		return(gecTooFewArgs);
	}

	//Validate pointers
	int i;
	for (i = 0; i < argCount; i++)
	{
		if (argv[i] == NULL)
		{
			printf("NULL argument found\r\n");
			return(gecNullArg);
		}
	}

	//Validate rotation order
	int found = 0;
	for (i = 0; i < rotordCount; i++)
	{
		if (strcmp(argv[argIdxRot], rotords[i]) == 0)
			found = 1;
	}
	if (found == 0)
	{
		printf("Invalid rotation order\r\n");
		return(gecInvRot);
	}

	//Validate X angle
	found = validateRotation(argv[argIdxXax]);
	if (found != 0)
	{
		printf("Invalid X angle\r\n");
		return(found);
	}

	//Validate Y angle
	found = validateRotation(argv[argIdxYax]);
	if (found != 0)
	{
		printf("Invalid Y angle\r\n");
		return(found);
	}

	//Validate Z angle
	found = validateRotation(argv[argIdxZax]);
	if (found != 0)
	{
		printf("Invalid Z angle\r\n");
		return(found);
	}

	//Validate JBEAM file name
	if (strlen(argv[argIdxJbeam]) <= 0)
	{
		printf("Zero-length JBEAM file name\r\n");
		return(gec0LenFile);
	}

	//Validate DAE file name
	if (strlen(argv[argIdxDae]) <= 0)
	{
		printf("Zero-length DAE file name\r\n");
		return(gec0LenFile);
	}

	//Bail
	return(gecNoError);
}

///////////////////
// Main Function //
///////////////////
int		main(
int		argc,
char*	argv[])
{
	//Validate arguments
	int	rtn = validateArgs(argc, argv);
	if (rtn != 0) return(rtn);
	
	//Get rotation angles
	double	xRot = atof(argv[argIdxXax]);
	double	yRot = atof(argv[argIdxYax]);
	double	zRot = atof(argv[argIdxZax]);
	double	xMov = atof(argv[argIdxXmv]);
	double	yMov = atof(argv[argIdxYmv]);
	double	zMov = atof(argv[argIdxZmv]);

	//Transform JBEAM file
	rtn = xFormJbeam(argv[argIdxJbeam], argv[argIdxRot], xRot, yRot, zRot, xMov, yMov, zMov);
	if (rtn != 0) return(rtn);

	//Transform DAE file
	rtn = xFormDae(argv[argIdxDae], argv[argIdxRot], xRot, yRot, zRot, xMov, yMov, zMov);
	return(rtn);
}
