//////////////////////////
// DAE File Transformer //
//////////////////////////

///////////////////
// Include Files //
///////////////////
#include	<stdio.h>
#include	<stdlib.h>
#include	<string.h>
#include	<ctype.h>
#include	"ErrorCodes.h"
#include	"FileNames.h"
#include	"Coord_Trans.h"
#include	"DAE_Transformer.h"

////////////////
// State enum //
////////////////
#define		flagPosSec	(0x01)
typedef	enum
{
	daeState_Idle= 0,
	daeState_ColTag,
	daeState_ColData,
	daeState_GotParm1,
	daeState_GotParm2
}
DaeStateEnum;

///////////////////////
// Parser state type //
///////////////////////
typedef struct
{
	char*			buf;
	FILE*			fDst;
	DaeStateEnum	ds;
	int				flags;
	const char*		sRotord;
	double			xRot;
	double			yRot;
	double			zRot;
	double			xMov;
	double			yMov;
	double			zMov;
	double			dX;
	double			dY;
	double			dZ;

	//Debug stuff
//	int				dbgParmCount;
}
DaeStateType,
*DaeStatePtr;

////////////////////////
// String has numbers //
////////////////////////
int					StrHasNum(
const char* const	str)
{
	//Return var
	int		rtn = 0;

	//Loop through characters
	int		sLen = (int)strlen(str);
	int		idx = 0;
	while (idx < sLen && rtn == 0)
	{
		if (isdigit(str[idx]))
			rtn = 1;
		idx++;
	}

	//Bail
	return(rtn);
}

/////////////////////////
// Output single float //
/////////////////////////
int					FloatSingleOut(
const double		dOut,
FILE* const			fDst)
{
	//Return var
	int		rtn = gecNoError;

	//Write number followed by space
	if (fprintf(fDst, "%#.8f", dOut) < 0)
		rtn = gecFileWrite;
	else if (fputc(' ', fDst) == EOF)
		rtn = gecFileWrite;

	//Bail
	return(rtn);
}

/////////////////////////
// Output float vector //
/////////////////////////
int					FloatVecOut(
const DaeStatePtr	s)
{
	//Write 3 doubles to file
	int		rtn = FloatSingleOut(s->dX, s->fDst);
	if(rtn == gecNoError)
		rtn = FloatSingleOut(s->dY, s->fDst);
	if (rtn == gecNoError)
		rtn = FloatSingleOut(s->dZ, s->fDst);
	return(rtn);
}

//////////////////////
// Transform coords //
//////////////////////
int					XformDaeCoords(
const DaeStatePtr	s)
{
	FloatVecType	v1 = {s->dX, s->dY, s->dZ};
	FloatVecType	v2 = { 0, 0, 0 };
	FloatVecType	vDist = {s->xRot, s->yRot, s->zRot};
	CoordRot(&v1, &v2, &vDist, s->sRotord);
	s->dX = v2.x;
	s->dY = v2.y;
	s->dZ = v2.z;
	if (s->flags & flagPosSec)
	{
		s->dX += s->xMov;
		s->dY += s->yMov;
		s->dZ += s->zMov;
	}

	//Write coordinates to file
	int		rtn = FloatVecOut(s);
	//s->dbgParmCount += 3;
	return(rtn);
}

///////////////////////////
// Data collection state //
///////////////////////////
int					DaeState_ColData(
const DaeStatePtr	s,
char				c)
{
	//Return var
	int		rtn = gecNoError;

	//Space character; Delimiter
	if (c == ' ')
	{
		//Collected at least some numbers
		if (StrHasNum(s->buf))
		{
			//Already got 1 parameter
			if (s->ds == daeState_GotParm1)
			{
				s->dY = atof(s->buf);
				s->ds = daeState_GotParm2;
			}

			//Already got 2 parameters
			else if (s->ds == daeState_GotParm2)
			{
				s->dZ = atof(s->buf);

				//////////////////////
				// Process numerics //
				//////////////////////
				rtn = XformDaeCoords(s);

				s->ds = daeState_ColData;
			}

			//No parameters collected yet
			else
			{
				s->dX = atof(s->buf);
				s->ds = daeState_GotParm1;
			}

			//Clear buffer
			s->buf[0] = '\0';
		}
	}

	//Start of next tag
	else if(c == '<')
	{
		//Fetch leftover parameter, if it completes a set
		if (StrHasNum(s->buf) && s->ds == daeState_GotParm2)
		{
			s->dZ = atof(s->buf);
			//////////////////////
			// Process numerics //
			//////////////////////
			rtn = XformDaeCoords(s);
		}
		rtn = CharFile(s->fDst, '<');

//		printf("Parm Count = %i\r\n", s->dbgParmCount);
		//Clear buffer & change state
		s->buf[0] = '\0';
		s->ds = daeState_ColTag;
	}

	//Some other char
	else
	{
		strcatc(s->buf, c);
	}

	//Bail
	return(rtn);
}

/////////////////
// Analyze tag //
/////////////////
int					AnalyzeTag(
const DaeStatePtr	s,
char				c)
{
	//Return var
	int		rtn = gecNoError;

	//Find "float_array" and ("mesh-positions-array" or "mesh-normals-array")
	int		isPosSec = cistrstr(s->buf, "mesh-positions-array");
	if (cistrstr(s->buf, "float_array") &&
	(isPosSec || cistrstr(s->buf, "mesh-normals-array")))
	{
		s->ds = daeState_ColData;
		if (isPosSec)
			s->flags |= flagPosSec;
		else
			s->flags &= ~flagPosSec;
//		printf("Parsing DAE section %s\r\n", s->buf);
//		s->dbgParmCount = 0;
	}
	else
	{
		s->ds = daeState_Idle;
	}

	//Bail
	s->buf[0] = '\0';
	return(rtn);
}

//////////////////////////
// Tag collection state //
//////////////////////////
int					DaeState_ColTag(
const DaeStatePtr	s,
char				c)
{
	//Write just read char to file
	int		rtn = CharFile(s->fDst, c);
	if (rtn == gecNoError)
	{
		//Found '>'; End of tag; Must analyze tag
		if (c == '>') AnalyzeTag(s, c);

		//Collect more chars
		else strcatc(s->buf, c);
	}
	//Bail
	return(rtn);
}

//////////////////////////////////
// Idle state; Looking for tags //
//////////////////////////////////
int					DaeState_Idle(
const DaeStatePtr	s,
char				c)
{
	//Write just read char to file
	int		rtn = CharFile(s->fDst, c);
	if (rtn == gecNoError)
	{
		//Found '<'; Start collecting tags
		if (c == '<') s->ds = daeState_ColTag;
	}
	//Bail
	return(rtn);
}

///////////////////////
// Process character //
///////////////////////
int					ProcChar(
const DaeStatePtr	s,
char				c)
{
	//Return var
	int		rtn = gecNoError;

	//Which state
	switch (s->ds)
	{
		case daeState_Idle:
			rtn = DaeState_Idle(s, c);
		break;
		case daeState_ColTag:
			rtn = DaeState_ColTag(s, c);
		break;
		case daeState_ColData:
		case daeState_GotParm1:
		case daeState_GotParm2:
			rtn = DaeState_ColData(s, c);
		break;
		default:
			s->ds = daeState_Idle;
			rtn = gecBadDaeNodeState;
			printf("DAE state error\r\n");
	}

	//Bail
	return(rtn);
}

////////////////////
// Parse DAE file //
////////////////////
int					ParseDae(
const DaeStatePtr	s,
FILE* const			fSrc)
{
	//Return var
	int		rtn = gecNoError;

	//Loop through all characters in source file
	int		c = EOF;
	do
	{
		c = fgetc(fSrc);
		if (c != EOF)
		{
			//Process character
			rtn = ProcChar(s, char(c));
		}
	}
	while (c != EOF && rtn == gecNoError);

	//Error reading file
	if (ferror(fSrc) != 0)
	{
		printf("Error reading from source file\r\n");
		rtn = gecFileRead;
	}

	//Bail
	return(rtn);
}

//////////////////////////
// Transform DAE file //
//////////////////////////
int					xFormDae(
const char* const	sDae,
const char* const	sRotord,
double				xRot,
double				yRot,
double				zRot,
double				xMov,
double				yMov,
double				zMov)
{
	//Return var
	int		rtn = gecNoError;

	//Allocate state structure
	DaeStatePtr		dState = DaeStatePtr(malloc(sizeof(DaeStateType)));
	if (dState == NULL)
	{
		printf("Cannot allocate state structure\r\n");
		rtn = gecMemAllocErr;
	}
	else
	{
		//Initialize state
		memset(dState, 0, sizeof(DaeStateType));
		dState->sRotord = (char*)sRotord;
		dState->xRot = xRot;
		dState->yRot = yRot;
		dState->zRot = zRot;
		dState->xMov = xMov;
		dState->yMov = yMov;
		dState->zMov = zMov;

		//Allocate primary buffer
		dState->buf = (char*)malloc(512);
		if (dState->buf == NULL)
		{
			printf("Cannot allocate primary DAE buffer\r\n");
			rtn = gecMemAllocErr;
		}
		else
		{
			//Initialize primary buffer
			memset(dState->buf, 0, 512);

			//Construct destination file name
			char	sDst[FILENAME_MAX];
			rtn = GetDestFileName(sDae, sDst);

			//Open source file
			FILE*	fSrc = fopen(sDae, "r");
			if (fSrc == NULL)
			{
				printf("Cannot open read file \"%s\"\r\n", sDae);
				rtn = gecFileNonexistent;
			}
			else
			{
				//Open destination file
				dState->fDst = fopen(sDst, "w");
				if (dState->fDst == NULL)
				{
					printf("Cannot open write file \"%s\"\r\n", sDst);
					return(gecFileCantMake);
				}
				else
				{
					//Parse DAE file
					rtn = ParseDae(dState, fSrc);

					//Close destination file
					fclose(dState->fDst);
					dState->fDst = NULL;
				}

				//Close source file
				fclose(fSrc);
				fSrc = NULL;
			}

			//Free memory
			free(dState->buf);
			dState->buf = NULL;
		}

		//Release memory
		free(dState);
		dState = NULL;
	}

	//Bail
	return(rtn);
}
