////////////////////////////
// File Name Manipulation //
////////////////////////////

///////////////////
// Include Files //
///////////////////
#include	<stdio.h>
#include	<stdlib.h>
#include	<string.h>
#include	<ctype.h>
#include	"FileNames.h"
#include	"ErrorCodes.h"

///////////////////////////////
// Get Destination File Name //
///////////////////////////////
int					GetDestFileName(
const char* const	sSrcFile,
char* const			sDstFile)
{
	//Find last backslash, if any
	char* bslash = strrchr((char*)sSrcFile, int('\\'));

	//Find last period, if any
	char* period = strrchr((char*)sSrcFile, int('.'));

	//Split string?
	if (period != NULL && bslash < period)
	{
		int		sLen = int(period - sSrcFile);
		strncpy(sDstFile, sSrcFile, sLen);
		sDstFile[sLen] = '\0';
		strcat(sDstFile, sDstSuffix);
		strcat(sDstFile, period);
	}
	else
	{
		strcpy(sDstFile, sSrcFile);
		strcat(sDstFile, sDstSuffix);
	}
	return(gecNoError);
}


/////////////////////////
// Get lower-case copy //
/////////////////////////
char*				GetLCCopy(
const char* const	pSrc,
int					sLen)
{
	char* pDst = NULL;
	if (pSrc != NULL && sLen >= 0)
	{
		pDst = (char*)malloc((size_t)sLen + 1);
		if (pDst != NULL)
		{
			strncpy(pDst, pSrc, (size_t)sLen);
			pDst[sLen] = '\0';
			for (int idx = 0; idx < sLen; idx++)
				pDst[idx] = (tolower(pDst[idx]));
		}
	}
	return(pDst);
}


//////////////////////////////
// Case-insensitive strncmp //
//////////////////////////////
int					cistrncmp(
const char* const	buf,
const char* const	trgt,
int					cnt)
{
	int		rtn = 0;

	//Get lower-case string copies
	char* cpyBuf = GetLCCopy(buf, cnt);
	if (cpyBuf != NULL)
	{
		char* cpyTrgt = GetLCCopy(trgt, (int)strlen(trgt));
		if (cpyTrgt != NULL)
		{
			//Find string
			if (strcmp(cpyBuf, cpyTrgt) == 0)
				rtn = 1;

			//Free memory & bail
			free(cpyTrgt);
		}
		free(cpyBuf);
	}
	return(rtn);
}

/////////////////////////////
// Case-insensitive strcmp //
/////////////////////////////
int					cistrcmp(
const char* const	buf,
const char* const	trgt)
{
	return(cistrncmp(buf, trgt, (int)strlen(buf)));
}



//////////////////////////////
// Case-insensitive strnstr //
//////////////////////////////
int					cistrnstr(
const char* const	buf,
const char* const	trgt,
int					cnt)
{
	int	rtn = 0;

	//Get lower-case string copies
	char* cpyBuf = GetLCCopy(buf, cnt);
	if (cpyBuf != NULL)
	{
		char* cpyTrgt = GetLCCopy(trgt, (int)strlen(trgt));
		if (cpyTrgt != NULL)
		{
			//Find string
			if (strstr(cpyBuf, cpyTrgt) != NULL)
				rtn = 1;

			//Free memory & bail
			free(cpyTrgt);
		}
		free(cpyBuf);
	}
	return(rtn);
}

/////////////////////////////
// Case-insensitive strstr //
/////////////////////////////
int					cistrstr(
const char* const	buf,
const char* const	trgt)
{
	return(cistrnstr(buf, trgt, (int)strlen(buf)));
}

/////////////////////////////////////
// Concatonate character to string //
/////////////////////////////////////
void				strcatc(
char* const			buf,
char				c)
{
	char	str[] = {c, '\0'};
	strcat(buf, str);
}

////////////////////////////////////
// Count chars of particular type //
////////////////////////////////////
int						charCount(
	const char* const		sChars,
	char					c)
{
	//Nothing to do
	if (sChars == NULL) return(0);
	int		sLen = (int)strlen(sChars);
	if (sLen <= 0) return(0);

	//Count chars
	int	cnt = 0;
	for (int idx = 0; idx < sLen; idx++)
		if (sChars[idx] == c) cnt++;
	return(cnt);
}

////////////////////////
// Put string in file //
////////////////////////
int					StrFile(
FILE*				fDst,
const char* const	str)
{
	return(fputs(str, fDst) == EOF ? gecFileWrite : gecNoError);
}

//////////////////////
// Put char in file //
//////////////////////
int					CharFile(
FILE* const			fDst,
const char			c)
{
	return(fputc(c, fDst) == EOF ? gecFileWrite : gecNoError);
}